/*
 * Ben NanoNote screen details and panel-specific functions.
 *
 * Copyright (C) Xiangfu Liu <xiangfu@sharism.cc>
 * Copyright (C) 2015, 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "board.h"
#include "jzlcd.h"
#include "nanonote.h"
#include "gpio.h"

struct jzfb_info jzfb = {
	.cfg=MODE_8BIT_SERIAL_TFT | PCLK_N | HSYNC_N | VSYNC_N,
	.w=320, .h=240,		/* dimensions */
	.bpp=32,		/* bits per pixel */
	.fclk=70,		/* frame clock rate */
	.hsw=1, .vsw=1,		/* sync widths */
	.elw=273, .blw=140,	/* line limits */
	.efw=1, .bfw=20		/* frame limits */
};

vidinfo_t panel_info = {
	.jz_fb=&jzfb,	// this will need correcting for user mode usage
	.lcd=0,		// base address for registers
};

/* Display configuration functions. */

static void spi_write_reg1(uint8_t reg, uint8_t val)
{
	uint8_t no;
	uint16_t value;
	void *gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_LCD);

	jz4740_gpio_set_pin(gpio_port_base, SPEN);
	jz4740_gpio_set_pin(gpio_port_base, SPCK);
	jz4740_gpio_clear_pin(gpio_port_base, SPDA);
	jz4740_gpio_clear_pin(gpio_port_base, SPEN);

	value = ((reg << 8) | (val & 0xFF));

	for (no = 0; no < 16; no++)
	{
		jz4740_gpio_clear_pin(gpio_port_base, SPCK);
		jz4740_gpio_set_value(gpio_port_base, SPDA, value & 0x8000 ? 1 : 0);
		jz4740_gpio_set_pin(gpio_port_base, SPCK);
		value = (value << 1);
	}

	jz4740_gpio_set_pin(gpio_port_base, SPEN);
}

void lcd_display_pin_init()
{
	void *gpio_port_base = jz4740_gpio_get_port((void *) GPIO_BASE, GPIO_PORT_LCD);

	jz4740_gpio_as_output(gpio_port_base, SPEN);
	jz4740_gpio_as_output(gpio_port_base, SPCK);
	jz4740_gpio_as_output(gpio_port_base, SPDA);
}

void lcd_display_on()
{
	spi_write_reg1(0x05, 0x1e);
	spi_write_reg1(0x05, 0x5e);
	spi_write_reg1(0x07, 0x8d);
	spi_write_reg1(0x13, 0x01);
	spi_write_reg1(0x05, 0x5f);
}

void lcd_display_off()
{
	spi_write_reg1(0x05, 0x5e);
}
